import { Request, Response } from "express";
import prisma, { Prisma } from "@server/prisma";
import { datatableRequest } from "@server/controllers/guild/api/dt.module";

export const get = async (request: Request, response: Response) => {
    if (!request.query.id) {
        response.status(400).json({ error: "missing 'id' query" });
        return;
    }

    const feed = await prisma.feed.findUnique({
        where: { id: Number(request.query.id) },
        include: { channels: true }
    });

    if (!feed) {
        response.status(404).json({ message: "no result found" });
        return;
    }

    response.json(feed);
};

export const post = async (request: Request, response: Response) => {
    const guildId = request.params.guildId;
    const { name, url, active, channels } = request.body;

    // channels comes through as either String[] or String
    const formattedChannels = Array.isArray(channels)
        ? channels.map((channelId) => ({ channel_id: channelId }))
        : [{ channel_id: channels }]

    let feed;

    try {
        feed = await prisma.feed.create({
            data: { 
                name: name,
                url: url,
                guild_id: guildId,
                active: active === "on",
                channels: channels !== undefined ? { create: formattedChannels } : channels
             }
        });
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(201).json(feed);
};

export const patch = async () => {}  // TODO ...

export const del = async (request: Request, response: Response) => {
    const { ids } = request.body;
    const guildId = request.params.guildId;

    if (!ids || !Array.isArray(ids)) {
        response.status(400).json({ error: "invalid request body" });
        return;
    }

    try {
        await prisma.feed.deleteMany({ where: {
            id: { in: ids },
            guild_id: guildId
        }});
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(204).json(null);
};

export const datatable = async (request: Request, response: Response) => {
    return await datatableRequest(
        request,
        response,
        prisma.feed,
        { id: "asc" },
        { channels: true, filters: true },
        { guild_id: request.params.guildId }  // TODO: verify authenticated user can access this guild
    );
};

export default { get, post, patch, del, datatable };